/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                 .NET C# Language Spatial SDK, Version 6.0                  */
/*                    Copyright 2022, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2022 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ANPPExample
{
    public partial class Form1 : Form
    {
        Socket client = null;
        Boolean networkConnected = false;
        Boolean serialConnected = false;
        ANPacketDecoder anPacketDecoder;

        public Form1()
        {
            InitializeComponent();
            anPacketDecoder = new ANPacketDecoder();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            comboSerialBaud.SelectedIndex = 7;
            updatePorts();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            updatePorts();
        }

        private void updatePorts()
        {
            if (!serialConnected)
            {
                string[] ports = System.IO.Ports.SerialPort.GetPortNames();
                Boolean portsChanged = ports.Length != comboSerialPort.Items.Count;
                if (!portsChanged)
                {
                    for(int i=0; i < ports.Length; i++)
                    {
                        if(ports[i].CompareTo(comboSerialPort.Items[i].ToString()) != 0)
                        {
                            portsChanged = true;
                            break;
                        }
                    }
                }
                if (portsChanged)
                {
                    comboSerialPort.Items.Clear();
                    foreach (string portName in ports)
                    {
                        comboSerialPort.Items.Add((object)portName);
                    }
                    if (ports.Length > 0)
                    {
                        comboSerialPort.SelectedIndex = 0;
                    }
                }
            }
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            if (serialConnected)
            {
                try
                {
                    serialPort1.Close();
                    buttonSerialConnect.Text = "Connect";
                    buttonNetworkConnect.Enabled = false;
                    serialConnected = false;
                }
                catch { }
            }
            else
            {
                try
                {
                    serialPort1.PortName = comboSerialPort.Text;
                    serialPort1.BaudRate = Convert.ToInt32(comboSerialBaud.Text);
                    serialPort1.Open();
                    buttonSerialConnect.Text = "Disconnect";
                    serialConnected = true;
                    buttonNetworkConnect.Enabled = true;
                }
                catch { }
            }
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            try
            {
                anPacketDecoder.bufferLength += serialPort1.Read(anPacketDecoder.buffer, anPacketDecoder.bufferLength, anPacketDecoder.buffer.Length - anPacketDecoder.bufferLength);
                decodePackets();
            }
            catch { }
        }

        private void networkReceiveCallback(IAsyncResult ar)
        {
            try
            {
                anPacketDecoder.bufferLength += client.EndReceive(ar);
                decodePackets();
                client.BeginReceive(anPacketDecoder.buffer, anPacketDecoder.bufferLength, anPacketDecoder.buffer.Length - anPacketDecoder.bufferLength, 0, new AsyncCallback(networkReceiveCallback), null);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
        }

        private void decodePackets()
        {
            ANPacket packet = null;
            while ((packet = anPacketDecoder.packetDecode()) != null)
            {
                switch (packet.id)
                {
                    case ANPacket.PACKET_ID_SYSTEM_STATE:
                        if (packet.length == 100)
                        {
                            ANPacket20 anPacket20 = new ANPacket20(packet);
                            this.Invoke((MethodInvoker)delegate
                            {
                                richTextBox1.AppendText("Received System State Packet\n");
                                textBoxLatitude.Text = (anPacket20.position[0] * 180 / Math.PI).ToString();
                                textBoxLongitude.Text = (anPacket20.position[1] * 180 / Math.PI).ToString();
                                textBoxHeight.Text = anPacket20.position[2].ToString();
                                textBoxRoll.Text = (anPacket20.orientation[0] * 180 / Math.PI).ToString();
                                textBoxPitch.Text = (anPacket20.orientation[1] * 180 / Math.PI).ToString();
                                textBoxYaw.Text = (anPacket20.orientation[2] * 180 / Math.PI).ToString();
                            });
                        }
                        break;
                    case ANPacket.PACKET_ID_RAW_SENSORS:
                        if (packet.length == 48)
                        {
                            ANPacket28 anPacket28 = new ANPacket28(packet);
                            this.Invoke((MethodInvoker)delegate
                            {
                                richTextBox1.AppendText("Received Raw Sensors Packet\n");
                            });
                        }
                        break;
                }
            }
        }

        private void buttonNetworkConnect_Click(object sender, EventArgs e)
        {
            if (!networkConnected)
            {
                try
                {
                    // Create a TCP/IP socket.  
                    client = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

                    // Connect to the remote endpoint.  
                    client.Connect(textHostName.Text, Convert.ToInt32(textTcpPort.Text));
                    client.BeginReceive(anPacketDecoder.buffer, anPacketDecoder.bufferLength, anPacketDecoder.buffer.Length - anPacketDecoder.bufferLength, 0, new AsyncCallback(networkReceiveCallback), null);

                    networkConnected = true;
                    buttonSerialConnect.Enabled = false;
                    buttonNetworkConnect.Text = "Disconnect";
                }
                catch (Exception exception)
                {
                    Console.WriteLine(exception.ToString());
                }
            }
            else
            {
                client.Shutdown(SocketShutdown.Both);
                client.Close();
                networkConnected = false;
                buttonSerialConnect.Enabled = true;
                buttonNetworkConnect.Text = "Connect";
            }
        }
    }
}
