/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Spatial SDK, Version 6.0                   */
/*                    Copyright 2022, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2022 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Raw Satellite Data */
public class ANPacket60
{
	public static final int SATELLITE_SYSTEM_UNKNOWN 	= 0;
	public static final int SATELLITE_SYSTEM_GPS 		= 1;
	public static final int SATELLITE_SYSTEM_GLONASS 	= 2;
	public static final int SATELLITE_SYSTEM_BEIDOU 	= 3;
	public static final int SATELLITE_SYSTEM_GALILEO 	= 4;
	public static final int SATELLITE_SYSTEM_SBAS 		= 5;
	public static final int SATELLITE_SYSTEM_QZSS 		= 6;
	public static final int SATELLITE_SYSTEM_STARFIRE 	= 7;
	public static final int SATELLITE_SYSTEM_OMNISTAR 	= 8;

	public long unixTime;
	public long nanoSeconds;
	public int receiverClockOffset;
	public int receiverNumber;
	public int packetNumber;
	public int totalPackets;
	public Satellite satellites[];

	public ANPacket60(ANPacket packet) throws IndexOutOfBoundsException
	{
		unixTime = TypeConversion.bytesToUInt32(packet.data, 0);
		nanoSeconds = TypeConversion.bytesToUInt32(packet.data, 4);
		receiverClockOffset = TypeConversion.bytesToInt32(packet.data, 8);
		receiverNumber = packet.data[12] & 0xFF;
		packetNumber = packet.data[13] & 0xFF;
		totalPackets = packet.data[14] & 0xFF;
		int numberSatellites = packet.data[15] & 0xFF;
		satellites = new Satellite[numberSatellites];
		for (int i = 0, j = 16; i < numberSatellites && j + 32 <= packet.length; i++)
		{
			satellites[i] = new Satellite();
			satellites[i].satelliteSystem = packet.data[j] & 0xFF;
			satellites[i].prn = packet.data[j + 1] & 0xFF;
			satellites[i].elevation = packet.data[j + 2];
			satellites[i].azimuth = TypeConversion.bytesToInt16(packet.data, j + 3);
			int numberFrequencies = packet.data[j + 5] & 0xFF;
			satellites[i].frequencies = new Frequency[numberFrequencies];
			j += 6;
			for (int k = 0; k < numberFrequencies && j + 26 <= packet.length; k++)
			{
				satellites[i].frequencies[k] = new Frequency();
				satellites[i].frequencies[k].frequency = packet.data[j] & 0xFF;
				satellites[i].frequencies[k].carrierPhaseValid = (packet.data[j + 1] & 0x01) > 0;
				satellites[i].frequencies[k].carrierPhaseCycleSlip = (packet.data[j + 1] & 0x02) > 0;
				satellites[i].frequencies[k].carrierPhaseHalfCycle = (packet.data[j + 1] & 0x04) > 0;
				satellites[i].frequencies[k].pseudoRangeValid = (packet.data[j + 1] & 0x08) > 0;
				satellites[i].frequencies[k].dopplerValid = (packet.data[j + 1] & 0x10) > 0;
				satellites[i].frequencies[k].snrValid = (packet.data[j + 1] & 0x20) > 0;
				satellites[i].frequencies[k].carrierPhase = TypeConversion.bytesToDouble(packet.data, j + 2);
				satellites[i].frequencies[k].pseudoRange = TypeConversion.bytesToDouble(packet.data, j + 10);
				satellites[i].frequencies[k].doppler = TypeConversion.bytesToFloat(packet.data, j + 18);
				satellites[i].frequencies[k].snr = TypeConversion.bytesToFloat(packet.data, j + 22);
				j += 26;
			}
			if(numberFrequencies > 0 && satellites[i].frequencies[numberFrequencies-1] == null)
			{
				satellites[i].frequencies = new Frequency[0];
			}
		}
		if(numberSatellites > 0 && satellites[numberSatellites-1] == null)
		{
            satellites = new Satellite[0];
        }
	}

	public class Satellite
	{
		public int satelliteSystem;
		public int prn;
		public int elevation;
		public int azimuth;
		public Frequency[] frequencies;

		public String satelliteSystemName()
		{
			switch(satelliteSystem)
			{
				case SATELLITE_SYSTEM_GPS:
					return "GPS";
				case SATELLITE_SYSTEM_GLONASS:
					return "GLONASS";
				case SATELLITE_SYSTEM_BEIDOU:
					return "BEIDOU";
				case SATELLITE_SYSTEM_GALILEO:
					return "GALILEO";
				case SATELLITE_SYSTEM_SBAS:
					return "SBAS";
				case SATELLITE_SYSTEM_QZSS:
					return "QZSS";
				case SATELLITE_SYSTEM_STARFIRE:
					return "Starfire";
				case SATELLITE_SYSTEM_OMNISTAR:
					return "Omnistar";
				default:
					return "Unknown";
			}
		}
	}

	public class Frequency
	{
		public int frequency;
		public Boolean carrierPhaseValid;
		public Boolean carrierPhaseCycleSlip;
		public Boolean carrierPhaseHalfCycle;
		public Boolean pseudoRangeValid;
		public Boolean dopplerValid;
		public Boolean snrValid;
		public double carrierPhase;
		public double pseudoRange;
		public float doppler;
		public float snr;
	}
}
